@@ -1,5 +1,5 @@
 package Treex::Core::TredView;
-$Treex::Core::TredView::VERSION = '1.20150902';
+$Treex::Core::TredView::VERSION = '2.20150928';
 # planned to be used from contrib.mac of tred's extensions
 
 use Moose;
@@ -597,9 +597,10 @@ sub anode_hint {
 
     # List all non-empty Interset features.
     if ( $node->does('Treex::Core::Node::Interset') ) {
-        my @iset = $node->get_iset_pairs_list();
-        for ( my $i = 0; $i <= $#iset; $i += 2 ) {
-            push @lines, "$iset[$i]: $iset[$i+1]";
+        my @ifeat = $node->iset()->get_nonempty_features();
+        foreach my $f ( @ifeat ) {
+            my $v = $node->iset()->get_joined($f);
+            push(@lines, "$f: $v");
         }
     }
 
@@ -881,7 +882,7 @@ Treex::Core::TredView - visualization of Treex files in TrEd
 
 =head1 VERSION
 
-version 1.20150902
+version 2.20150928
 
 =head1 DESCRIPTION
 
